#!/bin/bash

# Must be root
if [ $UID -ne 0 ]; then
    echo >&2 'ERROR: Not root.'
    exit 1
fi

# Our passwd file
passwdfile=/etc/passwd
backup_passwd=$passwdfile.bak-$(date +%F)
if [ ! -f $passwdfile ]; then
    echo >&2 "ERROR: File doesn't exist: $passwdfile"
    exit 1
fi

# Warnings array
warnings=()

# Don't do anything if no compat users found
if [ $(grep -c '^+' $passwdfile) -eq 0 ]; then
    echo 'No compat users found. Nothing to do.'
    exit 0
fi

# Make a backup of passwd file
cp -a $passwdfile $backup_passwd

# Access file we'll create
accessdir=/etc/security/access.d
accessfile=${accessdir}/20-compat-from-passwd-$(date +%F).conf

# Find entries in passwd, sorting users from groups
users=()
groups=()
for entry in $(grep -E '^\+[^:]' $passwdfile | tr -d '^+'); do
    # Do we have SSSD homedir override?
    if grep -q -E '^_homedir_override$' /etc/uio/info/classes.txt; then
	sssd_homedir_override=1
    else
	sssd_homedir_override=0
    fi

    # Check for empty or manageable passwd overrides
    if [[ $entry =~ ^([[:alnum:]@-]+)::::::$ ]]; then
	entry=${BASH_REMATCH[1]}
    elif [ $sssd_homedir_override -eq 1 ] && [[ $entry =~ ^([[:alnum:]-]+):::::/home/([[:alnum:]-]+):$ ]] && [ ${BASH_REMATCH[1]} == ${BASH_REMATCH[2]} ]; then
	entry=${BASH_REMATCH[1]}
    fi

    # Check for non-empty passwd overrides
    if [[ $entry =~ ^[[:alnum:]@-]+: ]]; then
	warnings+=( "Entry '+$entry' has unmanagable overrides and is IGNORED" )
	continue
    fi

    if [[ $entry =~ ^@ ]]; then
	groups+=( $entry )
    else
	users+=( $entry )
    fi
done

# Entries giving universal access
if grep -q -E '^\+:|^\+$' $passwdfile; then
    warnings+=( "Ignoring line(s) in passwd giving universal access" )
fi

# Create access file in /etc/security/access.d
if [ -e $accessfile ]; then
    echo >&2 "ERROR: File already exists: $accessfile"
    exit 1
else
    [ -d $accessdir ] || mkdir $accessdir
    cat <<EOF >$accessfile
#-------------------------------------------------------------
# Automatically created by $0
#-------------------------------------------------------------
EOF
    if [ ${#users[@]} -gt 0 ]; then
	echo >> $accessfile
	echo '# Access for users' >> $accessfile
	for user in ${users[@]}; do
	    echo "+: $user : ALL" >> $accessfile
	done
    fi
    if [ ${#groups[@]} -gt 0 ]; then
	echo >> $accessfile
	echo '# Access for netgroups' >> $accessfile
	for group in ${groups[@]}; do
	    echo "+: $group : ALL" >> $accessfile
	done
    fi
fi

# Delete compat lines in passwd
sed -i "/^\+/d" $passwdfile

# Print summary
echo 'SUMMARY:'
echo '================================================================='
echo
echo "* We have deleted all lines beginning with '+' in $passwdfile."
echo "  A backup is available as $backup_passwd"
echo "  Changes to $passwdfile:"
echo
diff -T $backup_passwd $passwdfile

if [ -e $accessfile ]; then
    echo
    echo '* Users and netgroups transferred to:'
    echo "    $accessfile"
fi

if [ ${#warnings[@]} -gt 0 ]; then
    echo
    echo 'PROBLEMS ENCOUNTERED:'
    echo '-----------------------------------------------------------------'
    IFS=''
    for w in ${warnings[@]}; do
	echo "  * $w"
    done
else
    echo
    echo '* No problems encountered'
fi
echo
